# puppetplugin

## Table of Contents

1. [Description](#description)
1. [Setup - The basics of getting started with puppetplugin](#setup)
    * [Setup requirements](#setup-requirements)
1. [Usage - Configuration options and additional functionality](#usage)
1. [Limitations - OS compatibility, etc.](#limitations)
1. [Development - Guide for contributing to the module](#development)

## Description

This module allows you to securely fetch secrets form **RevBits** PAM.

## Setup

### Setup Requirements

Install `aes-everywhere` gem on **puppet-server** & **puppet-agent** with the following command.

```ruby
/opt/puppetlabs/puppet/bin/gem install aes-everywhere
```

### Installation

To install this module, run the following command on the Puppet server:
```
puppet module install revbits-puppetplugin
```

To install a specific version of this module (e.g. `v1.0.0`), run the following
command on the Puppet server:

```
puppet module install revbits-puppetplugin --version 1.0.0
```

## Usage

### Example Usage

```puppet
$secret_value = Deferred(puppetplugin::secret, ['KEY_TO_FETCH', {
  appliance_url => "https://appliance-url.com",
  api_key => Sensitive("YourAPIKey")
}])

file { '/tmp/puppetfile': # Resource type file
  ensure => file, # Create as a file 
  owner => 'root', # Ownership
  group => 'root', # Group Name
  mode => '0644', # File permissions
  content => $secret_value
}
```

#### `Deferred` functions

This module leverages
[`Deferred` functions](https://puppet.com/docs/puppet/6.17/deferring_functions.html) introduced in puppet v6+
to make sure that all credentials are retrieved on agent side.

```ruby
Deferred(puppetplugin::secret, ['KEY_TO_FETCH', { #...options }])
```

> Note: The above puppet function will not work if not wrapped in `Deferred`

#### `Sensitive` type

The value returned from`puppetplugin::secret` is wrapped in `Sensitive` data type so that it is difficult to mishandle secrets accidentally.

> To get the value as `String` you will need to use `unwrap` on returned value i.e. `$secret_value.unwrap`

## Reference

For detailed references check [REFERENCE.md](https://github.com/RevBits/puppetplugin/blob/main/README.md)

## Limitations

See [metadata.json](https://github.com/RevBits/puppetplugin/blob/main/metadata.json) for dependencies and compatibility.

## Development

We at **RevBits** welcome all contributions.

### Contribution Guide

- Create a fork of this module's repository
- Add your contribution on a branch starting with issue number `e.g. 221-fixing-typo-in-docs`
- Create a pull request against upstream

We will review your work and possibly merge to `main`